<?php
/**
 * A filter for checking if the the supplied confirmation code is valid
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, this is free software
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class ConfirmCodeVerifierFilter extends BaseFilter implements Filterable {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);

		// Set part description
		$this->setObjectDescription("A filter for confirmation code verification");

		// Create unique ID number
		$this->generateUniqueId();
	}

	/**
	 * Creates an instance of this filter class
	 *
	 * @return	$filterInstance		An instance of this filter class
	 */
	public final static function createConfirmCodeVerifierFilter () {
		// Get a new instance
		$filterInstance = new ConfirmCodeVerifierFilter();

		// Return the instance
		return $filterInstance;
	}

	/**
	 * Executes the filter with given request and response objects
	 *
	 * @param	$requestInstance	An instance of a class with an Requestable interface
	 * @param	$responseInstance	An instance of a class with an Responseable interface
	 * @return	void
	 * @throws	NullPointerException	If the user instance from registry is null
	 */
	public function execute (Requestable $requestInstance, Responseable $responseInstance) {
		// Get confirmation code from request
		$confirmCode = $requestInstance->getRequestElement('confirm');

		// Is this code set?
		if (is_null($confirmCode)) {
			// Is not in request
			$requestInstance->requestIsValid(false);

			// Add a message to the response
			$responseInstance->addFatalMessage('confirm_code_unset');

			// Abort here
			return false;
		} elseif (empty($confirmCode)) {
			// Email is empty
			$requestInstance->requestIsValid(false);

			// Add a message to the response
			$responseInstance->addFatalMessage('confirm_code_empty');

			// Abort here
			return false;
		}

		// Get a user instance from registry
		$userInstance = Registry::getRegistry()->getInstance('user');

		// Is the instance there?
		if (is_null($userInstance)) {
			// Throw an exception here
			throw new NullPointerException ($this, self::EXCEPTION_IS_NULL_POINTER);
		} // END - if

		// Get the confirm code from user for comparison
		$userCode = $userInstance->getField(UserDatabaseWrapper::DB_COLUMN_CONFIRM_HASH);

		// Do we have the same code or different?
		if ($userCode != $confirmCode) {
			// Email is empty
			$requestInstance->requestIsValid(false);

			// Redirect to error page
			$responseInstance->redirectToConfiguredUrl('confirm_code_invalid_url');

			// Stop processing here
			exit;
		} // END - if
	}
}

// [EOF]
?>
